<?php

namespace Drupal\webform_telegram_notice\Plugin\WebformHandler;

use Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException;
use Drupal\Component\Plugin\Exception\PluginNotFoundException;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\webform\Annotation\WebformHandler;
use Drupal\webform\Entity\Webform;
use Drupal\webform\Plugin\WebformHandlerBase;
use Drupal\webform\WebformSubmissionInterface;
use GuzzleHttp\Exception\GuzzleException;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Webform handler.
 *
 * @WebformHandler(
 *   id = "telegram_notice_handler",
 *   label = @Translation("Telegram notice handler"),
 *   category = @Translation("Telegram notice"),
 *   description = @Translation("Sends the webform data to the telegram group."),
 *   cardinality = \Drupal\webform\Plugin\WebformHandlerInterface::CARDINALITY_SINGLE,
 *   results = \Drupal\webform\Plugin\WebformHandlerInterface::RESULTS_PROCESSED,
 * )
 */
class TelegramNoticeHandler extends WebformHandlerBase
{
  /**
   * The token manager.
   */
  protected $tokenManager;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition): TelegramNoticeHandler|WebformHandlerBase|ContainerFactoryPluginInterface
  {
    $instance = parent::create($container, $configuration, $plugin_id, $plugin_definition);
    $instance->tokenManager = $container->get('webform.token_manager');

    return $instance;
  }

  /**
   * {@inheritdoc}
   * @param array $form
   * @param FormStateInterface $form_state
   * @param WebformSubmissionInterface $webform_submission
   * @throws GuzzleException
   */
  public function submitForm(array &$form, FormStateInterface $form_state, WebformSubmissionInterface $webform_submission): void
  {
    $config = \Drupal::config('webform_telegram_notice.settings');
    $telegram_token = $config->get('token');
    $telegram_id_chat = $config->get('id_chat');
    $data = $form_state->getValues();
    $message = [];

    if($telegram_token && $telegram_id_chat) {
      if(isset($form['#webform_id']) && $form['#webform_id']) {

        // Getting label webform
        $webform = Webform::load($form['#webform_id']);
        if($webform) {
          $label = $webform->label();
          $message[] ='<b>'. strtoupper($label) .'</b>';
        }

        if($data) {
          // The exception is unnecessary fields for sending
          $exception = $config->get('exception');
          if($exception) {
            $exception = str_replace(' ','', $exception);
            $exception = explode(',', $exception);

            foreach ($exception as $value) {
              if(isset($data[$value])) {
                unset($data[$value]);
              }
            }
          }

          // Getting the field values
          foreach ($data as $key => $value) {
            if(!empty($value)) {
              $element = $webform->getElement($key);
              $title = '<b>' . $element['#title'] . ': </b>';

              $value = match ($element['#type']) {
                'webform_term_select' => $this->getTermNameByTid($form_state->getValue($key)),
                default => $form_state->getValue($key),
              };

              $message[] = $title . $value;
            }
          }
        }

        // Sending webform data to telegram
        $query = [
          'query' => [
            'chat_id' => $telegram_id_chat,
            'text' => implode("\r\n", $message),
            'parse_mode' => 'html',
          ]
        ];

        \Drupal::httpClient()->get("https://api.telegram.org/bot{$telegram_token}/sendMessage", $query);
      }
    }
  }

  /**
   * Getting the name of a term by TID.
   *
   * @param $tid
   * @return TranslatableMarkup|mixed|string|null
   */
  protected function getTermNameByTid($tid): mixed
  {
    $name = '';
    try {
      $term = \Drupal::entityTypeManager()->getStorage('taxonomy_term')->load($tid);
      $name = $term->label();
    }
    catch (InvalidPluginDefinitionException|PluginNotFoundException $e) {
      \Drupal::logger('webform_telegram_notice')->error($e);
    }

    return $name;
  }
}
